import sys

from PyQt5.QtWidgets import QDialog
from qtpy.QtGui import QFont
from qtpy.QtCore import Qt, Signal
from qtpy.QtWidgets import QApplication, QLabel, QHBoxLayout, QVBoxLayout, QFrame, QWidget
from qtpy.QtWidgets import QSplitter

from pyqt_openai.image_gen_widget.leftSideBar import LeftSideBar
from pyqt_openai.image_gen_widget.rightSideBar import RightSideBar
from pyqt_openai.image_gen_widget.viewWidget import ViewWidget
from pyqt_openai.inputDialog import InputDialog
from pyqt_openai.svgButton import SvgButton


class ImageGeneratingToolWidget(QWidget):
    notifierWidgetActivated = Signal()

    def __init__(self):
        super().__init__()
        self.__initUi()

    def __initUi(self):
        # TODO
        self.__leftSideBarWidget = LeftSideBar()
        self.__leftSideBarWidget.setVisible(False)
        self.__leftSideBarWidget.added.connect(self.__addImageGroup)
        self.__leftSideBarWidget.deleted.connect(self.__deleteImageGroup)
        # changed
        # imageUpdated
        self.__leftSideBarWidget.export.connect(self.__exportImageGroup)

        self.__viewWidget = ViewWidget()
        self.__rightSideBarWidget = RightSideBar()
        self.__rightSideBarWidget.notifierWidgetActivated.connect(self.notifierWidgetActivated)
        self.__rightSideBarWidget.submitDallE.connect(self.__setResult)
        self.__rightSideBarWidget.submitSd.connect(self.__setResult)

        self.__sideBarBtn = SvgButton()
        self.__sideBarBtn.setIcon('ico/sidebar.svg')
        self.__sideBarBtn.setCheckable(True)
        self.__sideBarBtn.setToolTip('Chat List')
        self.__sideBarBtn.setChecked(True)
        self.__sideBarBtn.toggled.connect(self.__leftSideBarWidget.setVisible)

        self.__historyBtn = SvgButton()
        self.__historyBtn.setIcon('ico/history.svg')
        self.__historyBtn.setCheckable(True)
        self.__historyBtn.setToolTip('History')
        self.__historyBtn.setChecked(True)
        self.__historyBtn.toggled.connect(self.__viewWidget.getExplorerWidget().setVisible)

        self.__settingBtn = SvgButton()
        self.__settingBtn.setIcon('ico/setting.svg')
        self.__settingBtn.setCheckable(True)
        self.__settingBtn.setToolTip('Settings')
        self.__settingBtn.setChecked(True)
        self.__settingBtn.toggled.connect(self.__rightSideBarWidget.setVisible)

        lay = QHBoxLayout()
        # lay.addWidget(self.__sideBarBtn)
        lay.addWidget(self.__settingBtn)
        lay.addWidget(self.__historyBtn)
        lay.setContentsMargins(2, 2, 2, 2)
        lay.setAlignment(Qt.AlignLeft)

        self.__menuWidget = QWidget()
        self.__menuWidget.setLayout(lay)
        self.__menuWidget.setMaximumHeight(self.__menuWidget.sizeHint().height())

        sep = QFrame()
        sep.setFrameShape(QFrame.HLine)
        sep.setFrameShadow(QFrame.Sunken)

        mainWidget = QSplitter()
        mainWidget.addWidget(self.__leftSideBarWidget)
        mainWidget.addWidget(self.__viewWidget)
        mainWidget.addWidget(self.__rightSideBarWidget)
        mainWidget.setSizes([100, 700, 200])
        mainWidget.setChildrenCollapsible(False)
        mainWidget.setHandleWidth(2)
        mainWidget.setStyleSheet(
        '''
        QSplitter::handle:horizontal
        {
            background: #CCC;
            height: 1px;
        }
        ''')

        lay = QVBoxLayout()
        lay.addWidget(self.__menuWidget)
        lay.addWidget(sep)
        lay.addWidget(mainWidget)
        lay.setContentsMargins(0, 0, 0, 0)
        lay.setSpacing(0)
        self.setLayout(lay)

    def showAiToolBar(self, f):
        self.__menuWidget.setVisible(f)

    def setAIEnabled(self, f):
        self.__rightSideBarWidget.setEnabled(f)

    def __addImageGroup(self):
        self.__db.insertConv('New Chat')
        cur_id = self.__db.getCursor().lastrowid
        self.__browser.resetChatWidget(cur_id)
        self.__leftSideBarWidget.addImageGroup(cur_id)
        self.__lineEdit.setFocus()
        print('addImageGroup')

    def __deleteImageGroup(self):
        print('deleteImageGroup')

    def __exportImageGroup(self):
        print('exportImageGroup')

    def __setResult(self, arg):
        # DALL-E
        if isinstance(arg, str):
            self.__viewWidget.showDallEResult(arg)
            # TODO
            # self.__leftSideBarWidget.addImageGroup('DALL-E', 'New Image Group', 0)
        # SD
        elif isinstance(arg, bytes):
            self.__viewWidget.showSdResult(arg)
            # self.__leftSideBarWidget.addImageGroup('Stable Diffusion', 'New Image Group', 0)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = ImageGeneratingToolWidget()
    w.show()
    sys.exit(app.exec_())



from qtpy.QtCore import QSettings
from qtpy.QtWidgets import QScrollArea, QWidget, QTabWidget, QGridLayout

from pyqt_openai.right_sidebar.chatPage import ChatPage
from pyqt_openai.right_sidebar.completionPage import CompletionPage
from pyqt_openai.sqlite import SqliteDatabase


class AIPlaygroundWidget(QScrollArea):
    def __init__(self, db: SqliteDatabase, ini_etc_dict, model_data):
        super().__init__()
        self.__initVal(db, ini_etc_dict, model_data)
        self.__initUi()

    def __initVal(self, db, ini_etc_dict, model_data):
        self.__db = db
        self.__modelData = model_data

        self.__ini_etc_dict = ini_etc_dict
        self.__settings_struct = QSettings('pyqt_openai.ini', QSettings.IniFormat)

        # load tab widget's last current index
        if self.__settings_struct.contains('TAB_IDX'):
            self.__cur_idx = int(self.__settings_struct.value('TAB_IDX'))
        else:
            self.__cur_idx = 0
            self.__settings_struct.setValue('TAB_IDX', str(self.__cur_idx))
        self.__db.setModelType(self.__cur_idx+1)

    def __initUi(self):
        tabWidget = QTabWidget()

        chatPage = ChatPage(self.__db, self.__ini_etc_dict)
        self.__completionPage = CompletionPage(self.__db, self.__ini_etc_dict, self.__modelData)

        tabWidget.addTab(chatPage, 'Chat', )
        tabWidget.addTab(self.__completionPage, 'Completion', )
        tabWidget.currentChanged.connect(self.__tabChanged)
        tabWidget.setCurrentIndex(self.__cur_idx)

        lay = QGridLayout()
        lay.addWidget(tabWidget)

        mainWidget = QWidget()
        mainWidget.setLayout(lay)

        self.setWidget(mainWidget)
        self.setWidgetResizable(True)

        self.setStyleSheet('QScrollArea { border: 0 }')

    def setModelInfoByModel(self, f: bool):
        self.__completionPage.setModelInfoByModel(f)

    def __tabChanged(self, idx):
        self.__settings_struct.setValue('TAB_IDX', idx)
        self.__db.setModelType(idx+1)


# from sqlalchemy import create_engine, Column, Integer, String
# from sqlalchemy.orm import sessionmaker, declarative_base
#
# # Create a SQLite database file named "myapp.db"
# engine = create_engine('sqlite:///myapp.db')
# Base = declarative_base()
# Session = sessionmaker(bind=engine)
# session = Session()
#
#
# # Define a model class
# class User(Base):
#     __tablename__ = 'users'
#     id = Column(Integer, primary_key=True)
#     name = Column(String)
#     email = Column(String)
#
#     def __repr__(self):
#         return f"<User(name='{self.name}', email='{self.email}')>"
#
#     def save(self):
#         session.add(self)
#         session.commit()
#
# # Create the database tables
# Base.metadata.create_all(engine)


from qtpy.QtWidgets import QMainWindow, QToolBar, QAction, QApplication


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        # Create two toolbar widgets
        toolbar1 = QToolBar("Toolbar 1")
        toolbar2 = QToolBar("Toolbar 2")

        # Add some actions to the toolbars
        action1 = QAction("Action 1", self)
        action2 = QAction("Action 2", self)
        action3 = QAction("Action 3", self)
        action4 = QAction("Action 4", self)

        toolbar1.addAction(action1)
        toolbar1.addAction(action2)

        toolbar2.addAction(action3)
        toolbar2.addAction(action4)

        # Add the toolbars to the main window
        self.addToolBar(toolbar1)
        self.addToolBar(toolbar2)



if __name__ == '__main__':
    app = QApplication([])
    window = MainWindow()
    window.show()
    app.exec_()



import sys
from qtpy.QtWidgets import QApplication, QMainWindow, QTextEdit, QVBoxLayout, QWidget, QCompleter
from qtpy.QtCore import Qt
from qtpy.QtGui import QTextCursor

class AutoCompleteTextEdit(QTextEdit):
    def __init__(self, parent=None):
        super(AutoCompleteTextEdit, self).__init__(parent)
        self.completer = None

    def set_completer(self, completer):
        if self.completer:
            self.completer.activated.disconnect()

        self.completer = completer
        self.completer.setWidget(self)
        self.completer.setCompletionMode(QCompleter.PopupCompletion)
        self.completer.activated.connect(self.insert_completion)

    def insert_completion(self, completion):
        tc = self.textCursor()
        tc.select(QTextCursor.WordUnderCursor)
        tc.insertText(completion)
        self.setTextCursor(tc)

    def text_under_cursor(self):
        tc = self.textCursor()
        tc.select(QTextCursor.WordUnderCursor)
        return tc.selectedText()

    def keyPressEvent(self, event):
        if self.completer and self.completer.popup().isVisible():
            if event.key() in (Qt.Key_Enter, Qt.Key_Return, Qt.Key_Escape, Qt.Key_Tab, Qt.Key_Backtab):
                event.ignore()
                return

        super(AutoCompleteTextEdit, self).keyPressEvent(event)

        if not self.completer:
            return

        if event.key() == Qt.Key_Slash:  # Activate completer when "/" is pressed
            completionPrefix = self.text_under_cursor()

            if completionPrefix != self.completer.completionPrefix():
                self.completer.setCompletionPrefix(completionPrefix)
                self.completer.popup().setCurrentIndex(self.completer.completionModel().index(0, 0))

            cr = self.cursorRect()
            cr.setWidth(self.completer.popup().sizeHintForColumn(
                0) + self.completer.popup().verticalScrollBar().sizeHint().width())
            self.completer.complete(cr)
        else:
            self.completer.popup().hide()

    # def keyPressEvent(self, event):
    #     if self.completer and self.completer.popup().isVisible():
    #         if event.key() in (Qt.Key_Enter, Qt.Key_Return, Qt.Key_Escape, Qt.Key_Tab, Qt.Key_Backtab):
    #             event.ignore()
    #             return
    #
    #     isShortcut = (event.modifiers() & Qt.ControlModifier) and event.key() == Qt.Key_E
    #     if not self.completer or not isShortcut:
    #         super(AutoCompleteTextEdit, self).keyPressEvent(event)
    #
    #     ctrlOrShift = event.modifiers() & (Qt.ControlModifier | Qt.ShiftModifier)
    #     if not self.completer or (ctrlOrShift and len(event.text()) == 0):
    #         return
    #
    #     eow = "~!@#$%^&*()_+{}|:\"<>?,./;'[]\\-="  # end of word characters
    #     hasModifier = (event.modifiers() != Qt.NoModifier) and not ctrlOrShift
    #     completionPrefix = self.text_under_cursor()
    #
    #     if not isShortcut and (hasModifier or len(event.text()) == 0 or len(completionPrefix) < 2 or event.text()[-1] in eow):
    #         self.completer.popup().hide()
    #         return
    #
    #     if completionPrefix != self.completer.completionPrefix():
    #         self.completer.setCompletionPrefix(completionPrefix)
    #         self.completer.popup().setCurrentIndex(self.completer.completionModel().index(0, 0))
    #
    #     cr = self.cursorRect()
    #     cr.setWidth(self.completer.popup().sizeHintForColumn(0) + self.completer.popup().verticalScrollBar().sizeHint().width())
    #     self.completer.complete(cr)

class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.text_edit = AutoCompleteTextEdit(self)
        self.init_autocomplete()

        layout = QVBoxLayout()
        layout.addWidget(self.text_edit)

        central_widget = QWidget(self)
        central_widget.setLayout(layout)
        self.setCentralWidget(central_widget)

    def init_autocomplete(self):
        words = ["apple", "banana", "grape", "application", "orange", "pineapple", "strawberry"]
        completer = QCompleter(words, self)
        self.text_edit.set_completer(completer)

if __name__ == "__main__":
    app = QApplication(sys.argv)
    mainWin = MainWindow()
    mainWin.show()
    sys.exit(app.exec_())



from qtpy.QtCore import Qt
from qtpy.QtGui import QPixmap, QPainter, QBitmap
from qtpy.QtWidgets import QLabel


class RoundedImage(QLabel):
    def __init__(self):
        super().__init__()
        self.__initVal()
        self.__initUi()

    def __initVal(self):
        self.__pixmap = ''
        self.__mask = ''

    def __initUi(self):
        pass

    def setImage(self, filename: str):
        # Load the image and set it as the pixmap for the label
        self.__pixmap = QPixmap(filename)
        self.__pixmap = self.__pixmap.scaled(self.__pixmap.width(), self.__pixmap.height(), Qt.KeepAspectRatio, Qt.SmoothTransformation)
        # Create a mask the same shape as the image
        self.__mask = QBitmap(self.__pixmap.size())

        # Create a QPainter to draw the mask
        self.__painter = QPainter(self.__mask)
        self.__painter.setRenderHint(QPainter.Antialiasing)
        self.__painter.setRenderHint(QPainter.SmoothPixmapTransform)
        self.__painter.fillRect(self.__mask.rect(), Qt.white)

        # Draw a black, rounded rectangle on the mask
        self.__painter.setPen(Qt.black)
        self.__painter.setBrush(Qt.black)
        self.__painter.drawRoundedRect(self.__pixmap.rect(), self.__pixmap.size().width(),
                                       self.__pixmap.size().height())
        self.__painter.end()

        # Apply the mask to the image
        self.__pixmap.setMask(self.__mask)
        self.setPixmap(self.__pixmap)
        self.setScaledContents(True)


import json
import os
import webbrowser

from qtpy.QtCore import Qt, QSettings, QEvent, Signal
from qtpy.QtGui import QCursor
from qtpy.QtWidgets import QHBoxLayout, QWidget, QSizePolicy, QVBoxLayout, QFrame, QSplitter, \
    QListWidgetItem, QFileDialog

from pyqt_openai.apiData import ModelData
from pyqt_openai.chatWidget import Prompt, ChatBrowser
from pyqt_openai.leftSideBar import LeftSideBar
from pyqt_openai.notifier import NotifierWidget
from pyqt_openai.openAiThread import OpenAIThread
from pyqt_openai.prompt.promptGeneratorWidget import PromptGeneratorWidget
from pyqt_openai.right_sidebar.aiPlaygroundWidget import AIPlaygroundWidget
from pyqt_openai.sqlite import SqliteDatabase
from pyqt_openai.svgButton import SvgButton


class OpenAIChatBotWidget(QWidget):
    notifierWidgetActivated = Signal()

    def __init__(self):
        super().__init__()
        self.__initVal()
        self.__initUi()

    def __initVal(self):
        # db
        self.__db = SqliteDatabase()

        # managing with ini file or something else
        self.__ini_etc_dict = {}

        self.__remember_past_conv = False
        self.__finishReason = False
        self.__modelData = ModelData()

        self.__ini_etc_dict['remember_past_conv'] = self.__remember_past_conv
        self.__ini_etc_dict['finishReason'] = self.__finishReason
        self.__ini_etc_dict['modelData'] = self.__modelData

        self.__settings_struct = QSettings('pyqt_openai.ini', QSettings.IniFormat)

        # make it compatible with version which was used json file as a database
        if self.__isConvHistoryJsonExists():
            self.__migrateJsonToSqlite()

        # "remember past conv" feature
        if self.__settings_struct.contains('REMEMBER_PAST_CONVERSATION'):
            self.__remember_past_conv = True if self.__settings_struct.value(
                'REMEMBER_PAST_CONVERSATION') == '1' else False
        else:
            self.__settings_struct.setValue('REMEMBER_PAST_CONVERSATION', '0')

        # don't care about this - just saving past conversation in gpt 3 and below
        if os.path.exists('conv.json'):
            pass
        else:
            with open('conv.json', 'w') as f:
                json.dump({}, f)

    def __isConvHistoryJsonExists(self):
        return os.path.exists('conv_history.json')

    def __migrateJsonToSqlite(self):
        self.__db.convertJsonIntoSql()
        os.remove('conv_history.json')

    def __initUi(self):
        self.__leftSideBarWidget = LeftSideBar()
        self.__browser = ChatBrowser()
        self.__prompt = Prompt(self.__db)
        self.__lineEdit = self.__prompt.getTextEdit()
        self.__aiPlaygroundWidget = AIPlaygroundWidget(self.__db, self.__ini_etc_dict, self.__modelData)
        self.__promptGeneratorWidget = PromptGeneratorWidget(self.__db)

        self.__sideBarBtn = SvgButton()
        self.__sideBarBtn.setIcon('ico/sidebar.svg')
        self.__sideBarBtn.setCheckable(True)
        self.__sideBarBtn.setToolTip('Chat List')
        self.__sideBarBtn.setChecked(True)
        self.__sideBarBtn.toggled.connect(self.__leftSideBarWidget.setVisible)

        self.__settingBtn = SvgButton()
        self.__settingBtn.setIcon('ico/setting.svg')
        self.__settingBtn.setToolTip('Settings')
        self.__settingBtn.setCheckable(True)
        self.__settingBtn.setChecked(True)
        self.__settingBtn.setChecked(False)
        self.__settingBtn.toggled.connect(self.__aiPlaygroundWidget.setVisible)

        self.__promptBtn = SvgButton()
        self.__promptBtn.setIcon('ico/prompt.svg')
        self.__promptBtn.setToolTip('Prompt Generator')
        self.__promptBtn.setCheckable(True)
        self.__promptBtn.setChecked(True)
        self.__promptBtn.setChecked(False)
        self.__promptBtn.toggled.connect(self.__promptGeneratorWidget.setVisible)

        lay = QHBoxLayout()
        lay.addWidget(self.__sideBarBtn)
        lay.addWidget(self.__settingBtn)
        lay.addWidget(self.__promptBtn)
        lay.setContentsMargins(2, 2, 2, 2)
        lay.setAlignment(Qt.AlignLeft)

        self.__menuWidget = QWidget()
        self.__menuWidget.setLayout(lay)
        self.__menuWidget.setMaximumHeight(self.__menuWidget.sizeHint().height())

        sep = QFrame()
        sep.setFrameShape(QFrame.HLine)
        sep.setFrameShadow(QFrame.Sunken)

        self.__leftSideBarWidget.initHistory(self.__db)
        self.__leftSideBarWidget.added.connect(self.__addConv)
        self.__leftSideBarWidget.changed.connect(self.__changeConv)
        self.__leftSideBarWidget.deleted.connect(self.__deleteConv)
        self.__leftSideBarWidget.convUpdated.connect(self.__updateConv)
        self.__leftSideBarWidget.export.connect(self.__export)

        self.__lineEdit.returnPressed.connect(self.__chat)

        self.__browser.convUnitUpdated.connect(self.__updateConvUnit)

        lay = QHBoxLayout()
        lay.addWidget(self.__prompt)
        lay.setSpacing(0)
        lay.setContentsMargins(0, 0, 0, 0)

        self.__queryWidget = QWidget()
        self.__queryWidget.setLayout(lay)
        self.__queryWidget.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Maximum)

        lay = QVBoxLayout()
        lay.addWidget(self.__browser)
        lay.addWidget(self.__queryWidget)
        lay.setSpacing(0)
        lay.setContentsMargins(0, 0, 0, 0)

        chatWidget = QWidget()
        chatWidget.setLayout(lay)

        self.__rightSideBar = QSplitter()
        self.__rightSideBar.setOrientation(Qt.Vertical)
        self.__rightSideBar.addWidget(self.__aiPlaygroundWidget)
        self.__rightSideBar.addWidget(self.__promptGeneratorWidget)
        self.__rightSideBar.setSizes([600, 400])
        self.__rightSideBar.setChildrenCollapsible(False)
        self.__rightSideBar.setHandleWidth(2)
        self.__rightSideBar.setStyleSheet(
            '''
            QSplitter::handle:vertical
            {
                background: #CCC;
                height: 1px;
            }
            ''')


        mainWidget = QSplitter()
        mainWidget.addWidget(self.__leftSideBarWidget)
        mainWidget.addWidget(chatWidget)
        mainWidget.addWidget(self.__rightSideBar)
        mainWidget.setSizes([100, 500, 400])
        mainWidget.setChildrenCollapsible(False)
        mainWidget.setHandleWidth(2)
        mainWidget.setStyleSheet(
        '''
        QSplitter::handle:horizontal
        {
            background: #CCC;
            height: 1px;
        }
        ''')

        lay = QVBoxLayout()
        lay.addWidget(self.__menuWidget)
        lay.addWidget(sep)
        lay.addWidget(mainWidget)
        lay.setContentsMargins(0, 0, 0, 0)
        lay.setSpacing(0)
        self.setLayout(lay)

        self.__lineEdit.setFocus()

    def setModelInfoByModel(self, f):
        self.__aiPlaygroundWidget.setModelInfoByModel(f)

    def showAiToolBar(self, f):
        self.__menuWidget.setVisible(f)

    def toolTipLinkClicked(self, url):
        webbrowser.open(url)

    def setAIEnabled(self, f):
        self.__lineEdit.setEnabled(f)

    def __chat(self):
        info_dict = self.__db.selectInfo()
        openai_arg = ''
        if self.__remember_past_conv:
            convs = []
            with open('conv.json', 'r') as f:
                for line in f:
                    conv = json.loads(line.strip())
                    convs.append(conv)
        # TODO refactoring
        if info_dict['engine'] in ['gpt-3.5-turbo', 'gpt-3.5-turbo-0301', 'gpt-4']:
            # "assistant" below is for making the AI remember the last question
            openai_arg = {
                'model': info_dict['engine'],
                'messages': [
                    {"role": "system", "content": info_dict['system']},
                    {"role": "assistant", "content": self.__browser.getAllText()},
                    {"role": "user", "content": self.__prompt.getContent()},
                ],
                # 'temperature': info_dict['temperature'],

                # won't use max_tokens, this is set to infinite by default
                # and i can't find any reason why should i limit the tokens currently
                # https://platform.openai.com/docs/api-reference/chat/create
                # 'max_tokens': self.__max_tokens,

                # 'top_p': info_dict['top_p'],
                # 'frequency_penalty': info_dict['frequency_penalty'],
                # 'presence_penalty': info_dict['presence_penalty'],

                'stream': info_dict['stream'],
            }
        else:
            openai_arg = info_dict
        if self.__leftSideBarWidget.isCurrentConvExists():
            pass
        else:
            self.__addConv()

        self.__lineEdit.setEnabled(False)
        self.__leftSideBarWidget.setEnabled(False)

        self.__browser.showLabel(self.__prompt.getContent(), True, False)

        self.__t = OpenAIThread(info_dict['engine'], openai_arg, self.__remember_past_conv)
        self.__t.replyGenerated.connect(self.__browser.showLabel)
        self.__t.streamFinished.connect(self.__browser.streamFinished)
        self.__lineEdit.clear()
        self.__t.start()
        self.__t.finished.connect(self.__afterGenerated)

    def __afterGenerated(self):
        self.__lineEdit.setEnabled(True)
        self.__leftSideBarWidget.setEnabled(True)
        self.__lineEdit.setFocus()
        if not self.isVisible():
            self.__notifierWidget = NotifierWidget(informative_text='Response 👌', detailed_text='Click this!')
            self.__notifierWidget.show()
            self.__notifierWidget.doubleClicked.connect(self.notifierWidgetActivated)

    def __changeConv(self, item: QListWidgetItem):
        # If a 'change' event occurs but there are no items, it should mean that list is empty
        # so reset conv_history.json
        if item:
            id = item.data(Qt.UserRole)
            conv = self.__db.selectConvUnit(id)
            self.__browser.replaceConv(id, conv)
        else:
            self.__browser.resetChatWidget(0)


    def __addConv(self):
        self.__db.insertConv('New Chat')
        cur_id = self.__db.getCursor().lastrowid
        self.__browser.resetChatWidget(cur_id)
        self.__leftSideBarWidget.addToList(cur_id)
        self.__lineEdit.setFocus()

    def __updateConv(self, id, title=None):
        if title:
            self.__db.updateConv(id, title)


    def __deleteConv(self, id_lst):
        for id in id_lst:
            self.__db.deleteConv(id)


    def __export(self, ids):
        filename = QFileDialog.getSaveFileName(self, 'Save', os.path.expanduser('~'), 'SQLite DB file (*.db)')
        if filename[0]:
            filename = filename[0]
            self.__db.export(ids, filename)


    def __updateConvUnit(self, id, user_f, conv_unit=None):
        if conv_unit:
            self.__db.insertConvUnit(id, user_f, conv_unit)


import sys

from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QTableView, QHeaderView, QAbstractItemView, QTableWidget, QTableWidgetItem
from qtpy.QtWidgets import QApplication, QMainWindow, QTextEdit, QVBoxLayout, QWidget, QCompleter
from qtpy.QtCore import Qt
from qtpy.QtGui import QTextCursor

from pyqt_openai.sqlite import SqliteDatabase


class AutoCompleteTextEdit(QTextEdit):
    def __init__(self, parent=None):
        super(AutoCompleteTextEdit, self).__init__(parent)
        self.completer = None

    def set_completer(self, completer):
        if self.completer:
            self.completer.activated.disconnect()

        self.completer = completer
        self.completer.setWidget(self)
        self.completer.setCompletionMode(QCompleter.PopupCompletion)

    def text_under_cursor(self):
        tc = self.textCursor()
        tc.select(QTextCursor.WordUnderCursor)
        return tc.selectedText()

    def keyPressEvent(self, event):
        if self.completer and self.completer.popup().isVisible():
            if event.key() in (Qt.Key_Enter, Qt.Key_Return, Qt.Key_Escape, Qt.Key_Tab, Qt.Key_Backtab):
                event.ignore()
                return

        super(AutoCompleteTextEdit, self).keyPressEvent(event)

        if not self.completer:
            return

        if event.key() == Qt.Key_Slash:  # Activate completer when "/" is pressed
            completionPrefix = self.text_under_cursor()

            if completionPrefix != self.completer.completionPrefix():
                self.completer.setCompletionPrefix(completionPrefix)
                self.completer.popup().setCurrentIndex(self.completer.completionModel().index(0, 0))

            cr = self.cursorRect()
            cr.setWidth(self.completer.popup().sizeHintForColumn(
                0) + self.completer.popup().verticalScrollBar().sizeHint().width())
            self.completer.complete(cr)
        else:
            self.completer.popup().hide()

    # def keyPressEvent(self, event):
    #     if self.completer and self.completer.popup().isVisible():
    #         if event.key() in (Qt.Key_Enter, Qt.Key_Return, Qt.Key_Escape, Qt.Key_Tab, Qt.Key_Backtab):
    #             event.ignore()
    #             return
    #
    #     isShortcut = (event.modifiers() & Qt.ControlModifier) and event.key() == Qt.Key_E
    #     if not self.completer or not isShortcut:
    #         super(AutoCompleteTextEdit, self).keyPressEvent(event)
    #
    #     ctrlOrShift = event.modifiers() & (Qt.ControlModifier | Qt.ShiftModifier)
    #     if not self.completer or (ctrlOrShift and len(event.text()) == 0):
    #         return
    #
    #     eow = "~!@#$%^&*()_+{}|:\"<>?,./;'[]\\-="  # end of word characters
    #     hasModifier = (event.modifiers() != Qt.NoModifier) and not ctrlOrShift
    #     completionPrefix = self.text_under_cursor()
    #
    #     if not isShortcut and (hasModifier or len(event.text()) == 0 or len(completionPrefix) < 2 or event.text()[-1] in eow):
    #         self.completer.popup().hide()
    #         return
    #
    #     if completionPrefix != self.completer.completionPrefix():
    #         self.completer.setCompletionPrefix(completionPrefix)
    #         self.completer.popup().setCurrentIndex(self.completer.completionModel().index(0, 0))
    #
    #     cr = self.cursorRect()
    #     cr.setWidth(self.completer.popup().sizeHintForColumn(0) + self.completer.popup().verticalScrollBar().sizeHint().width())
    #     self.completer.complete(cr)

class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.__db = SqliteDatabase()

        self.text_edit = AutoCompleteTextEdit()
        self.init_autocomplete()

        layout = QVBoxLayout()
        layout.addWidget(self.text_edit)

        central_widget = QWidget(self)
        central_widget.setLayout(layout)
        self.setCentralWidget(central_widget)

    def init_autocomplete(self):
        self.__p_grp = []
        for group in self.__db.selectPropPromptGroup():
            p_grp_attr = [attr for attr in self.__db.selectPropPromptAttribute(group[0])]
            p_grp_value = ''
            for attr_obj in p_grp_attr:
                name = attr_obj[2]
                value = attr_obj[3]
                if value and value.strip():
                    p_grp_value += f'{name}: {value}\n'
            self.__p_grp.append({'name': group[1], 'value': p_grp_value})

        self.__t_grp = [{'name': obj[1], 'value': obj[2]} for obj in self.__db.selectTemplatePrompt()]

        print('Property Group: ', self.__p_grp)
        print('Template: ', self.__t_grp)

        # only name
        p_grp_nm_arr = [obj['name'] for obj in self.__p_grp]
        t_grp_nm_arr = [obj['name'] for obj in self.__t_grp]
        words = p_grp_nm_arr+t_grp_nm_arr

        completer = QCompleter(words)
        completer.activated.connect(self.__activated)
        self.text_edit.set_completer(completer)

    def __activated(self, word):
        for item in self.__p_grp:
            if item['name'] == word:
                v = item['value']
                break
        else:
            # Handle the case when 'b' is not found
            v = ''
        self.text_edit.textCursor().deletePreviousChar()
        self.text_edit.insertPlainText(v)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    mainWin = MainWindow()
    mainWin.show()
    sys.exit(app.exec_())



import os
import requests

from qtpy.QtCore import Qt, QPointF, Signal
from qtpy.QtGui import QPixmap, QColor, QBrush, QLinearGradient
from qtpy.QtWidgets import QGraphicsScene, QGraphicsPixmapItem, QGraphicsView, QApplication, QWidget, QHBoxLayout, QFileDialog, QCheckBox, \
    QGraphicsProxyWidget

from pyqt_openai.svgButton import SvgButton


class ThumbnailView(QGraphicsView):
    clicked = Signal(QPixmap)

    def __init__(self):
        super().__init__()
        self.__initVal()
        self.__initUi()

    def __initVal(self):
        self._scene = QGraphicsScene()
        self._p = QPixmap()
        self._item = QGraphicsPixmapItem()
        self.__aspectRatioMode = Qt.KeepAspectRatio

    def __initUi(self):
        self.__setControlWidget()

        # set mouse event
        # to make buttons appear and apply gradient
        # above the top of an image when you hover the mouse cursor over it
        self.setMouseTracking(True)
        self.__defaultBrush = self.foregroundBrush()
        gradient = QLinearGradient(QPointF(0, 0), QPointF(0, self.viewport().height()))
        gradient.setColorAt(0, QColor(0, 0, 0, 200))
        gradient.setColorAt(1, QColor(0, 0, 0, 0))
        self.__brush = QBrush(gradient)

        self.setMinimumSize(150, 150)

    def __setControlWidget(self):
        # copy the image
        copyBtn = SvgButton()
        copyBtn.setIcon('ico/copy_light.svg')
        copyBtn.clicked.connect(self.__copy)

        # download the image
        saveBtn = SvgButton()
        saveBtn.setIcon('ico/save_light.svg')
        saveBtn.clicked.connect(self.__save)

        lay = QHBoxLayout()
        lay.addWidget(copyBtn)
        lay.addWidget(saveBtn)

        self.__controlWidget = QWidget(self)
        self.__controlWidget.setLayout(lay)

        self.__controlWidget.hide()

    def __refreshSceneAndView(self):
        self._item = self._scene.addPixmap(self._p)
        self.fitInView(self.sceneRect(), self.__aspectRatioMode)
        self.setScene(self._scene)

    def setFilename(self, filename: str):
        self._scene = QGraphicsScene()
        self._p = QPixmap(filename)
        self.__refreshSceneAndView()

    def setUrl(self, url):
        self._scene = QGraphicsScene()
        response = requests.get(url)
        self._p.loadFromData(response.content)
        self.__refreshSceneAndView()
        return response.content

    def setContent(self, content):
        self._scene = QGraphicsScene()
        self._p.loadFromData(content)
        self.__refreshSceneAndView()

    def setPixmap(self, pixmap):
        self._scene = QGraphicsScene()
        self._p = pixmap
        self.__refreshSceneAndView()

    def setAspectRatioMode(self, mode):
        self.__aspectRatioMode = mode

    def __copy(self):
        QApplication.clipboard().setPixmap(self._p)

    def __save(self):
        filename = QFileDialog.getSaveFileName(self, 'Save', os.path.expanduser('~'), 'Image file (*.png)')
        if filename[0]:
            filename = filename[0]
            self._p.save(filename)

    def enterEvent(self, e):
        # Show the button when the mouse enters the view
        if self._item.pixmap().width():
            self.__controlWidget.move(self.rect().x(), self.rect().y())
            self.setForegroundBrush(self.__brush)
            self.__controlWidget.show()
        return super().enterEvent(e)

    def leaveEvent(self, e):
        # Hide the button when the mouse leaves the view
        self.__controlWidget.hide()
        self.setForegroundBrush(self.__defaultBrush)
        return super().leaveEvent(e)

    def resizeEvent(self, e):
        if self._item.pixmap().width():
            self.fitInView(self.sceneRect(), self.__aspectRatioMode)
            self.setScene(self._scene)
        return super().resizeEvent(e)

    def mousePressEvent(self, e):
        self.clicked.emit(self._p)
        return super().mousePressEvent(e)


import inspect
import json

import openai

from qtpy.QtCore import QThread, Signal

from pyqt_openai.apiData import getModelEndpoint


class OpenAIThread(QThread):
    """
    == replyGenerated Signal ==
    First: response
    Second: user or AI
    Third: streaming a chat completion or not
    """
    replyGenerated = Signal(str, bool, bool)
    streamFinished = Signal()

    def __init__(self, model, openai_arg, remember_f, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__model = model
        self.__endpoint = getModelEndpoint(model)
        self.__openai_arg = openai_arg
        self.__remember_f = remember_f

    def run(self):
        try:
            if self.__endpoint == '/v1/chat/completions':
                response = openai.ChatCompletion.create(
                       **self.__openai_arg
                )
                # if it is streaming, type will be generator
                if inspect.isgenerator(response):
                    for chunk in response:
                        delta = chunk['choices'][0]['delta']
                        response_text = delta.get('content', '')
                        if response_text:
                            self.replyGenerated.emit(response_text, False, True)
                        else:
                            finish_reason = chunk['choices'][0].get('finish_reason', '')
                            if finish_reason:
                                self.streamFinished.emit()
                else:
                    response_text = response['choices'][0]['message']['content']
                    self.replyGenerated.emit(response_text, False, False)
            elif self.__endpoint == '/v1/completions':
                openai_object = openai.Completion.create(
                    **self.__openai_arg
                )

                response_text = openai_object['choices'][0]['text'].strip()

                # this doesn't store any data, so we manually do that every time
                if self.__remember_f:
                    conv = {
                        'prompt': self.__openai_arg['prompt'],
                        'response': response_text,
                    }

                    with open('conv.json', 'a') as f:
                        f.write(json.dumps(conv) + '\n')

                self.replyGenerated.emit(response_text, False, False)
        except openai.error.InvalidRequestError as e:
            print(e)
            self.replyGenerated.emit('<p style="color:red">Your request was rejected as a result of our safety system.<br/>'
                                     'Your prompt may contain text that is not allowed by our safety system.</p>', False, False)
        except openai.error.RateLimitError as e:
            self.replyGenerated.emit(f'<p style="color:red">{e}<br/>Check the usage: https://platform.openai.com/account/usage<br/>Update to paid account: https://platform.openai.com/account/billing/overview', False, False)


import platform, subprocess

from qtpy.QtGui import QFont
from qtpy.QtWidgets import QWidget, QComboBox, QTextEdit, QLabel, QVBoxLayout, QApplication, QCheckBox, QDoubleSpinBox, \
    QSpinBox, QFormLayout, QHBoxLayout, QFileDialog, QPushButton, QLineEdit, QGroupBox
from qtpy.QtCore import Qt, QSettings

from pyqt_openai.apiData import getCompletionModel, getLatestModel
from pyqt_openai.modelTable import ModelTable
from pyqt_openai.sqlite import SqliteDatabase
from pyqt_openai.svgLabel import SvgLabel


class CompletionPage(QWidget):
    def __init__(self, db: SqliteDatabase, ini_etc_dict, model_data):
        super().__init__()
        self.__initVal(db, ini_etc_dict, model_data)
        self.__initUi()

    def __setChatInfo(self, db):
        self.__db = db

        self.__info_dict = self.__db.selectInfo(2)

    def __initVal(self, db, ini_etc_dict, model_data):
        self.__setChatInfo(db)
        self.__settings_struct = QSettings('pyqt_openai.ini', QSettings.IniFormat)

        self.__modelData = model_data
        self.__ini_etc_dict = ini_etc_dict

    def __initUi(self):
        self.__modelComboBox = QComboBox()
        self.__modelComboBox.addItems(getCompletionModel())
        self.__modelComboBox.setCurrentText(self.__info_dict['engine'])
        self.__modelComboBox.currentTextChanged.connect(self.__modelChanged)

        self.__fineTuningBtn = QPushButton('Fine Tuning')
        self.__fineTuningBtn.clicked.connect(self.__fineTuning)

        # find the training data
        self.__findDataLineEdit = QLineEdit()

        seeEveryModelCheckBox = QCheckBox('View every model (not all models may work)')
        seeEveryModelCheckBox.toggled.connect(self.__seeEveryModelToggled)
        seeEveryModelCheckBoxLbl = SvgLabel()
        seeEveryModelCheckBoxLbl.setSvgFile('ico/help.svg')
        seeEveryModelCheckBoxLbl.setToolTip(
            '''Check this box to show all models, including obsolete ones. \nIf you don\'t check this, combobox lists latest models.''')

        lay = QHBoxLayout()
        lay.addWidget(seeEveryModelCheckBox)
        lay.addWidget(seeEveryModelCheckBoxLbl)
        lay.setContentsMargins(0, 0, 0, 0)
        lay.setAlignment(Qt.AlignLeft)

        seeEveryModelWidget = QWidget()
        seeEveryModelWidget.setLayout(lay)

        temperatureSpinBox = QDoubleSpinBox()
        temperatureSpinBox.setRange(0, 1)
        temperatureSpinBox.setAccelerated(True)
        temperatureSpinBox.setSingleStep(0.01)
        temperatureSpinBox.setValue(self.__info_dict['temperature'])
        temperatureSpinBox.valueChanged.connect(self.__temperatureChanged)

        maxTokensSpinBox = QSpinBox()
        maxTokensSpinBox.setRange(0, 4000)
        maxTokensSpinBox.setAccelerated(True)
        maxTokensSpinBox.setValue(self.__info_dict['max_tokens'])
        maxTokensSpinBox.valueChanged.connect(self.__maxTokensChanged)

        toppSpinBox = QDoubleSpinBox()
        toppSpinBox.setRange(0, 1)
        toppSpinBox.setAccelerated(True)
        toppSpinBox.setSingleStep(0.01)
        toppSpinBox.setValue(self.__info_dict['top_p'])
        toppSpinBox.valueChanged.connect(self.__toppChanged)

        frequencyPenaltySpinBox = QDoubleSpinBox()
        frequencyPenaltySpinBox.setRange(0, 2)
        frequencyPenaltySpinBox.setAccelerated(True)
        frequencyPenaltySpinBox.setSingleStep(0.01)
        frequencyPenaltySpinBox.setValue(self.__info_dict['frequency_penalty'])
        frequencyPenaltySpinBox.valueChanged.connect(self.__frequencyPenaltyChanged)

        presencePenaltySpinBox = QDoubleSpinBox()
        presencePenaltySpinBox.setRange(0, 2)
        presencePenaltySpinBox.setAccelerated(True)
        presencePenaltySpinBox.setSingleStep(0.01)
        # presencePenaltySpinBox.setValue(self.__info_dict['presence_penalty'])
        # presencePenaltySpinBox.valueChanged.connect(self.__presencePenaltyChanged)

        modelTableSubLbl = QLabel('You can view the information only by entering the valid API key.')
        modelTableSubLbl.setFont(QFont('Arial', 9))

        self.__apiCheckPreviewLbl = QLabel('')
        self.__modelTable = ModelTable()

        lay = QVBoxLayout()
        lay.addWidget(modelTableSubLbl)
        lay.addWidget(self.__modelTable)

        modelTableGrpBox = QGroupBox()
        modelTableGrpBox.setTitle('Model Info (testing)')
        modelTableGrpBox.setLayout(lay)

        lay = QFormLayout()
        lay.addWidget(seeEveryModelWidget)
        lay.addRow('Model', self.__modelComboBox)
        lay.addRow(modelTableGrpBox)
        lay.addRow('Temperature', temperatureSpinBox)
        lay.addRow('Maximum length', maxTokensSpinBox)
        lay.addRow('Top P', toppSpinBox)
        lay.addRow('Frequency penalty', frequencyPenaltySpinBox)
        lay.addRow('Presence penalty', presencePenaltySpinBox)

        modelOptionGrpBox = QGroupBox()
        modelOptionGrpBox.setTitle('Model')
        modelOptionGrpBox.setLayout(lay)

        rememberPastConversationChkBox = QCheckBox('Store Previous Conversation in Real Time (testing)')
        rememberPastConversationChkBox.setChecked(self.__ini_etc_dict['remember_past_conv'])
        rememberPastConversationChkBox.setDisabled(True)
        rememberPastConversationChkBox.toggled.connect(self.__rememberPastConversationChkBoxToggled)

        lay = QVBoxLayout()
        lay.addWidget(rememberPastConversationChkBox)

        generalOptionGrpBox = QGroupBox()
        generalOptionGrpBox.setTitle('General')
        generalOptionGrpBox.setLayout(lay)

        lay = QVBoxLayout()
        lay.addWidget(modelOptionGrpBox)
        lay.addWidget(generalOptionGrpBox)
        lay.setAlignment(Qt.AlignTop)

        optionGrpBox = QGroupBox()
        optionGrpBox.setTitle('Option')
        optionGrpBox.setLayout(lay)

        # find the training data
        self.__findDataLineEdit = QLineEdit()

        findDataBtn = QPushButton('Find...')
        findDataBtn.clicked.connect(self.__findData)

        lay = QHBoxLayout()
        lay.setSpacing(0)
        lay.addWidget(self.__findDataLineEdit)
        lay.addWidget(findDataBtn)
        lay.setAlignment(Qt.AlignTop)
        lay.setContentsMargins(5, 5, 5, 1)

        fineTuneGrpBoxTopWidget = QWidget()
        fineTuneGrpBoxTopWidget.setLayout(lay)

        lay = QVBoxLayout()
        lay.addWidget(self.__fineTuningBtn)
        lay.setAlignment(Qt.AlignTop)

        fineTuneGrpBoxBottomWidget = QWidget()
        fineTuneGrpBoxBottomWidget.setLayout(lay)
        lay.setContentsMargins(5, 1, 5, 5)

        lay = QVBoxLayout()
        lay.addWidget(fineTuneGrpBoxTopWidget)
        lay.addWidget(fineTuneGrpBoxBottomWidget)
        lay.setAlignment(Qt.AlignTop)
        lay.setSpacing(0)
        lay.setContentsMargins(0, 0, 0, 0)

        fineTuneGrpBox = QGroupBox()
        fineTuneGrpBox.setTitle('Fine-tune training (coming soon)')
        fineTuneGrpBox.setLayout(lay)

        lay = QVBoxLayout()
        lay.addWidget(optionGrpBox)
        lay.addWidget(fineTuneGrpBox)

        self.setLayout(lay)

    def __temperatureChanged(self, v):
        self.__temperature = round(v, 2)
        self.__db.updateInfo(2, 'temperature', v)

    def __maxTokensChanged(self, v):
        self.__max_tokens = round(v, 2)
        self.__db.updateInfo(2, 'max_tokens', v)

    def __toppChanged(self, v):
        self.__topp = round(v, 2)
        self.__db.updateInfo(2, 'top_p', v)

    def __frequencyPenaltyChanged(self, v):
        self.__frequency_penalty = round(v, 2)
        self.__db.updateInfo(2, 'frequency_penalty', v)

    def __presencePenaltyChanged(self, v):
        self.__presence_penalty = round(v, 2)
        self.__db.updateInfo(2, 'presence_penalty', v)

    def __seeEveryModelToggled(self, f):
        curModel = self.__modelComboBox.currentText()
        self.__modelComboBox.currentTextChanged.disconnect(self.__modelChanged)
        self.__modelComboBox.clear()
        if f:
            self.__modelComboBox.addItems([model.id for model in self.__modelData.getModelData()])
        else:
            self.__modelComboBox.addItems(getCompletionModel())
        self.__modelComboBox.currentTextChanged.connect(self.__modelChanged)
        self.__modelComboBox.setCurrentText(curModel)

    # for fine-tuning, showing the detailed information of the model
    def setModelInfoByModel(self, init_model: bool = False):
        if init_model:
            self.__modelData.setModelData()
        self.__modelTable.setModelInfo(self.__modelData.getModelData(), self.__info_dict['engine'], 'allow_fine_tuning')
        self.__fineTuningBtn.setEnabled(self.__modelTable.getModelInfo())

    def __findData(self):
        filename = QFileDialog.getOpenFileName(self, 'Open', '', 'JSONL Files (*.jsonl)')
        if filename[0]:
            filename = filename[0]
            self.__findDataLineEdit.setText(filename)
            self.__fineTuningBtn.setEnabled(True)

    def __fineTuning(self):
        if platform.system() == 'Windows':
            subprocess.Popen('cmd.exe', creationflags=subprocess.CREATE_NEW_CONSOLE)
        elif platform.system() in ['Darwin', 'Linux']:
            subprocess.Popen('bash', creationflags=subprocess.CREATE_NEW_CONSOLE)

        # https://platform.openai.com/docs/guides/fine-tuning/cli-data-preparation-tool
        # validating & giving suggestions and reformat the data
        # subprocess.run('openai tools fine_tunes.prepare_data -f data.jsonl')

        # https://platform.openai.com/docs/guides/fine-tuning/create-a-fine-tuned-model
        # create a fine-tuned model
        # subprocess.run('openai api fine_tunes.create -t [TRAIN_FILE_ID_OR_PATH] -m [BASE_MODEL]')

        # run this when event stream is interrupted for any reason
        # subprocess.run('openai api fine_tunes.follow -i [YOUR_FINE_TUNE_JOB_ID]')
        # you can see the job done when it is finished
        # https://platform.openai.com/account/usage
        # https://platform.openai.com/playground

        # list the jobs
        # subprocess.run('openai api fine_tunes.list')

        # get the status of certain job. The resulting object includes
        # job status (which can be one of pending, running, succeeded, or failed)
        # and other information
        # subprocess.run('openai api fine_tunes.get -i [YOUR_FINE_TUNE_JOB_ID]')

        # cancel the job
        # subprocess.run('openai api fine_tunes.cancel -i [YOUR_FINE_TUNE_JOB_ID]')

    def __modelChanged(self, v):
        self.__info_dict['engine'] = v
        self.setModelInfoByModel()
        self.__db.updateInfo(2, 'engine', v)

    def __rememberPastConversationChkBoxToggled(self, f):
        self.__settings_struct.setValue('REMEMBER_PAST_CONVERSATION', str(int(f)))


from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

setup(
    name='pyqt-openai',
    version='0.1.5',
    author='Jung Gyu Yoon',
    author_email='yjg30737@gmail.com',
    license='MIT',
    packages=find_packages(),
    package_data={'pyqt_openai.ico': ['close.svg', 'openai.svg', 'discord.svg', 'github.svg', 'help.svg', 'customize.svg', 'history.svg',
                                      'user.svg', 'sidebar.svg', 'prompt.svg', 'save.svg', 'stackontop.svg',
                                      'add.svg', 'delete.svg', 'setting.svg', 'search.svg',
                                      'vertical_three_dots.svg']},
    description='PyQt OpenAI example',
    url='https://github.com/yjg30737/pyqt-openai.git',
    long_description_content_type='text/markdown',
    long_description=long_description,
    install_requires=[
        'PyQt5>=5.14',
        'PySide6',
        'qtpy',
        'aiohttp',
        'openai',
        'pyperclip',
        'stability_sdk'
    ]
)


from qtpy.QtCore import Qt, QUrl
from qtpy.QtGui import QPixmap, QDesktopServices
from qtpy.QtWidgets import QDialog, QPushButton, QHBoxLayout, QWidget, QVBoxLayout, QLabel

from pyqt_openai.svgLabel import SvgLabel


class ClickableLabel(SvgLabel):
    def __init__(self):
        super().__init__()
        self.__url = '127.0.0.1'

    def setUrl(self, url):
        self.__url = url

    def mouseReleaseEvent(self, QMouseEvent):
        if QMouseEvent.button() == Qt.LeftButton:
            QDesktopServices.openUrl(QUrl(self.__url))


class AboutDialog(QDialog):
    def __init__(self):
        super().__init__()
        self.__initUi()

    def __initUi(self):
        self.setWindowTitle('About')
        self.setWindowFlags(Qt.Window | Qt.WindowCloseButtonHint)

        self.__okBtn = QPushButton('OK')
        self.__okBtn.clicked.connect(self.accept)

        p = QPixmap('pyqtopenai.png')
        logoLbl = QLabel()
        logoLbl.setPixmap(p)

        expWidget = QLabel()
        expWidget.setText('''
        <h1>pyqt-openai</h1>
        <p>Powered by qtpy</p>
        ''')
        expWidget.setAlignment(Qt.AlignTop)

        self.__githubLbl = ClickableLabel()
        self.__githubLbl.setSvgFile('ico/github.svg')
        self.__githubLbl.setUrl('https://github.com/yjg30737/pyqt-openai')

        self.__discordLbl = ClickableLabel()
        self.__discordLbl.setSvgFile('ico/discord.svg')
        self.__discordLbl.setUrl('https://discord.gg/cHekprskVE')
        self.__discordLbl.setFixedSize(22, 19)

        lay = QHBoxLayout()
        lay.addWidget(logoLbl)
        lay.addWidget(expWidget)
        lay.addWidget(self.__githubLbl)
        lay.addWidget(self.__discordLbl)

        topWidget = QWidget()
        topWidget.setLayout(lay)

        cancelBtn = QPushButton('Cancel')
        cancelBtn.clicked.connect(self.close)

        lay = QHBoxLayout()
        lay.addWidget(self.__okBtn)
        lay.addWidget(cancelBtn)
        lay.setAlignment(Qt.AlignRight)
        lay.setContentsMargins(0, 0, 0, 0)

        okCancelWidget = QWidget()
        okCancelWidget.setLayout(lay)

        lay = QVBoxLayout()
        lay.addWidget(topWidget)
        lay.addWidget(okCancelWidget)

        self.setLayout(lay)




from qtpy.QtGui import QBrush, QColor
from qtpy.QtWidgets import QTableWidget, QHeaderView, QTableWidgetItem, QAbstractItemView
from qtpy.QtCore import Qt


class ModelTable(QTableWidget):
    def __init__(self):
        super().__init__()
        self.__initUi()

    def __initUi(self):
        self.setColumnCount(2)
        self.setRowCount(10)
        self.resizeColumnsToContents()
        self.horizontalHeader().setVisible(False)
        self.verticalHeader().setVisible(False)
        self.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        item1 = QTableWidgetItem('Allow Fine Tuning')
        item1.setTextAlignment(Qt.AlignCenter)
        item1.setBackground(QBrush(QColor(230, 230, 230)))
        item2 = QTableWidgetItem('N/A')
        item2.setTextAlignment(Qt.AlignCenter)
        self.setItem(0, 0, item1)
        self.setItem(0, 1, item2)
        self.setEditTriggers(QAbstractItemView.NoEditTriggers)

    # TODO set more attributes
    # currently set the allow_fine_tuning only
    def setModelInfo(self, model_list, model_name, prop):
        token = [model['permission'][0][prop] for model in model_list if model['id'] == model_name]
        # check the desired prop exists
        if len(token) > 0:
            token = token[0]
        else:
            token = 'N/A'
        if isinstance(token, bool):
            token = 'Yes' if token else 'No'
        item = QTableWidgetItem(token)
        item.setTextAlignment(Qt.AlignCenter)
        self.setItem(0, 1, item)

    # currently get the allow_fine_tuning only
    def getModelInfo(self):
        token = self.item(0, 1).text()
        if token in ['Yes', 'No', 'N/A']:
            token = True if token == 'Yes' else False
        return token


import sys
from qtpy.QtWidgets import QLabel, QApplication, QWidget, QVBoxLayout


def create_hyperlink_label(text, link):
    label = QLabel()
    label.setText(
        '''
        <html>
    <head>
        <style>
            pre { line-height: 100%; }
td.linenos .normal { color: inherit; background-color: transparent; padding-left: 5px; padding-right: 5px; }
span.linenos { color: inherit; background-color: transparent; padding-left: 5px; padding-right: 5px; }
td.linenos .special { color: #000000; background-color: #ffffc0; padding-left: 5px; padding-right: 5px; }
span.linenos.special { color: #000000; background-color: #ffffc0; padding-left: 5px; padding-right: 5px; }
.highlight .hll { background-color: #ffffcc }
.highlight { background: #ffffff; }
.highlight .c { color: #888888 } /* Comment */
.highlight .err { color: #FF0000; background-color: #FFAAAA } /* Error */
.highlight .k { color: #008800; font-weight: bold } /* Keyword */
.highlight .o { color: #333333 } /* Operator */
.highlight .ch { color: #888888 } /* Comment.Hashbang */
.highlight .cm { color: #888888 } /* Comment.Multiline */
.highlight .cp { color: #557799 } /* Comment.Preproc */
.highlight .cpf { color: #888888 } /* Comment.PreprocFile */
.highlight .c1 { color: #888888 } /* Comment.Single */
.highlight .cs { color: #cc0000; font-weight: bold } /* Comment.Special */
.highlight .gd { color: #A00000 } /* Generic.Deleted */
.highlight .ge { font-style: italic } /* Generic.Emph */
.highlight .gr { color: #FF0000 } /* Generic.Error */
.highlight .gh { color: #000080; font-weight: bold } /* Generic.Heading */
.highlight .gi { color: #00A000 } /* Generic.Inserted */
.highlight .go { color: #888888 } /* Generic.Output */
.highlight .gp { color: #c65d09; font-weight: bold } /* Generic.Prompt */
.highlight .gs { font-weight: bold } /* Generic.Strong */
.highlight .gu { color: #800080; font-weight: bold } /* Generic.Subheading */
.highlight .gt { color: #0044DD } /* Generic.Traceback */
.highlight .kc { color: #008800; font-weight: bold } /* Keyword.Constant */
.highlight .kd { color: #008800; font-weight: bold } /* Keyword.Declaration */
.highlight .kn { color: #008800; font-weight: bold } /* Keyword.Namespace */
.highlight .kp { color: #003388; font-weight: bold } /* Keyword.Pseudo */
.highlight .kr { color: #008800; font-weight: bold } /* Keyword.Reserved */
.highlight .kt { color: #333399; font-weight: bold } /* Keyword.Type */
.highlight .m { color: #6600EE; font-weight: bold } /* Literal.Number */
.highlight .s { background-color: #fff0f0 } /* Literal.String */
.highlight .na { color: #0000CC } /* Name.Attribute */
.highlight .nb { color: #007020 } /* Name.Builtin */
.highlight .nc { color: #BB0066; font-weight: bold } /* Name.Class */
.highlight .no { color: #003366; font-weight: bold } /* Name.Constant */
.highlight .nd { color: #555555; font-weight: bold } /* Name.Decorator */
.highlight .ni { color: #880000; font-weight: bold } /* Name.Entity */
.highlight .ne { color: #FF0000; font-weight: bold } /* Name.Exception */
.highlight .nf { color: #0066BB; font-weight: bold } /* Name.Function */
.highlight .nl { color: #997700; font-weight: bold } /* Name.Label */
.highlight .nn { color: #0e84b5; font-weight: bold } /* Name.Namespace */
.highlight .nt { color: #007700 } /* Name.Tag */
.highlight .nv { color: #996633 } /* Name.Variable */
.highlight .ow { color: #000000; font-weight: bold } /* Operator.Word */
.highlight .w { color: #bbbbbb } /* Text.Whitespace */
.highlight .mb { color: #6600EE; font-weight: bold } /* Literal.Number.Bin */
.highlight .mf { color: #6600EE; font-weight: bold } /* Literal.Number.Float */
.highlight .mh { color: #005588; font-weight: bold } /* Literal.Number.Hex */
.highlight .mi { color: #0000DD; font-weight: bold } /* Literal.Number.Integer */
.highlight .mo { color: #4400EE; font-weight: bold } /* Literal.Number.Oct */
.highlight .sa { background-color: #fff0f0 } /* Literal.String.Affix */
.highlight .sb { background-color: #fff0f0 } /* Literal.String.Backtick */
.highlight .sc { color: #0044DD } /* Literal.String.Char */
.highlight .dl { background-color: #fff0f0 } /* Literal.String.Delimiter */
.highlight .sd { color: #DD4422 } /* Literal.String.Doc */
.highlight .s2 { background-color: #fff0f0 } /* Literal.String.Double */
.highlight .se { color: #666666; font-weight: bold; background-color: #fff0f0 } /* Literal.String.Escape */
.highlight .sh { background-color: #fff0f0 } /* Literal.String.Heredoc */
.highlight .si { background-color: #eeeeee } /* Literal.String.Interpol */
.highlight .sx { color: #DD2200; background-color: #fff0f0 } /* Literal.String.Other */
.highlight .sr { color: #000000; background-color: #fff0ff } /* Literal.String.Regex */
.highlight .s1 { background-color: #fff0f0 } /* Literal.String.Single */
.highlight .ss { color: #AA6600 } /* Literal.String.Symbol */
.highlight .bp { color: #007020 } /* Name.Builtin.Pseudo */
.highlight .fm { color: #0066BB; font-weight: bold } /* Name.Function.Magic */
.highlight .vc { color: #336699 } /* Name.Variable.Class */
.highlight .vg { color: #dd7700; font-weight: bold } /* Name.Variable.Global */
.highlight .vi { color: #3333BB } /* Name.Variable.Instance */
.highlight .vm { color: #996633 } /* Name.Variable.Magic */
.highlight .il { color: #0000DD; font-weight: bold } /* Literal.Number.Integer.Long */
        </style>
    </head>
    <body>
        <div class="highlight"><pre><span></span><span class="k">def</span> <span class="nf">hello_world</span><span class="p">():</span>
    <span class="nb">print</span><span class="p">(</span><span class="s2">&quot;Hello, world!&quot;</span><span class="p">)</span>
</pre></div>

    </body>
</html>

        '''
    )
    label.setOpenExternalLinks(True)
    return label


app = QApplication(sys.argv)
window = QWidget()
layout = QVBoxLayout()

hyperlink_label = create_hyperlink_label("Click me!", "https://www.example.com")
layout.addWidget(hyperlink_label)

window.setLayout(layout)
window.show()
sys.exit(app.exec_())
# hyperlink
# from pygments import highlight
# from pygments.lexers import PythonLexer
# from pygments.formatters import HtmlFormatter
#
# code = """
# def hello_world():
#     print("Hello, world!")
# """
#
# lexer = PythonLexer()
# formatter = HtmlFormatter(style='colorful')
#
# css_styles = formatter.get_style_defs('.highlight')
#
# html_code = f"""
# <html>
#     <head>
#         <style>
#             {css_styles}
#         </style>
#     </head>
#     <body>
#         {highlight(code, lexer, formatter)}
#     </body>
# </html>
# """
#
# print(html_code)
#




import torch
from diffusers import StableDiffusionPipeline
from torch import autocast
import streamlit as st
from PIL import Image, ImageEnhance
import pandas as pd
import numpy as np

class StableDiffusionLoader:
    """
    Stable Diffusion loader and generator class. 

    Utilises the stable diffusion models from the `Hugging Face`(https://huggingface.co/spaces/stabilityai/stable-diffusion) library

    Attributes
    ----------
    prompt : str
        a text prompt to use to generate an associated image
    pretrain_pipe : str
        a pretrained image diffusion pipeline i.e. CompVis/stable-diffusion-v1-4

    """
    def __init__(self, 
                prompt:str, 
                pretrain_pipe:str='CompVis/stable-diffusion-v1-4'):
        """
        Constructs all the necessary attributes for the diffusion class.

        Parameters
        ----------
            prompt : str
                the prompt to generate the model
            pretrain_pipe : str
                the name of the pretrained pipeline
        """
        self.prompt = prompt
        self.pretrain_pipe = pretrain_pipe
        print(torch.cuda.is_available())
        self.device = "cuda" if torch.cuda.is_available() else "cpu"

        if self.device == 'cpu':
            raise MemoryError('GPU need for inference')

        assert isinstance(self.prompt, str), 'Please enter a string into the prompt field'
        assert isinstance(self.pretrain_pipe, str), 'Please use value such as `CompVis/stable-diffusion-v1-4` for pretrained pipeline'


    def generate_image_from_prompt(self, save_location='prompt.jpg', use_token=False,
                                   verbose=False):

        pipe = StableDiffusionPipeline.from_pretrained(
            self.pretrain_pipe, 
            revision="fp16", torch_dtype=torch.float16, 
            use_auth_token=use_token
            )
        pipe = pipe.to(self.device)
        with autocast(self.device):
            image = pipe(self.prompt)[0][0]
        image.save(save_location)
        if verbose: 
            print(f'[INFO] saving image to {save_location}')
        return image    

    def __str__(self) -> str:
        return f'[INFO] Generating image for prompt: {self.prompt}'

    def __len__(self):
        return len(self.prompt)

if __name__ == '__main__':
    import os
    SAVE_LOCATION = 'prompt.jpg'

    sd = StableDiffusionLoader('toy store')
    sd.generate_image_from_prompt(save_location=SAVE_LOCATION)

    # Open and display the image on the site
    image = Image.open(SAVE_LOCATION)